$set 12 #Newalias
$ #Usage
1	Usage: %s [-g]\n
$ #NoAlloc
2	Unable to allocate space for alias buffer!
$ #UpdateSystem
3	Updating the system alias file...
$ #UpdatePersonal
4	Updating your personal alias file...
$ #NoHOME
5	I'm confused - no HOME variable in environment!
$ #NoOpenIn
6	Couldn't open %s for input!
$ #NoOpendbz
7	Couldn't open %s.pag or %s.dir for output!
$ #NoOpenOut
8	Couldn't open %s for output!
$ #NoSave
9	** Not saving tables!  Please fix and re-run!
$ #Processed
10	processed %d aliases.\n
$ #LineToLong
11	Line + continuations exceeds maximum length of %ld:
$ #Split
12	Line too long, split using continuation line format (starting line\nwith whitespace):\n%.40s\n
$ #NoFieldSep
13	Error - alias "%.40s" missing '=' field separator.
$ #CharNotSupported
14	Error - character '%c' in alias '%s' is not supported.
$ #DupAlias
15	** Duplicate alias '%s' in file.  Multiples ignored.
$ #ErrWrite
16	** Error writing alias '%s'.
$ #ListUsage
17	Usage: listalias [ -s | -u ] <optional-regular-expression>\n
$ #NoPipe
18	cannot open pipe to egrep program for expressions!\n
$ #NoOpenData
19	Could not open %s data file!\n
$ #System
20	system
$ #User
21	user
$ #SplitShort
22	Line too long, split using continuation line format:
$ #AliasWSNotAllowed
23	Error - whitespace in alias '%.30s' is not allowed.
$ #AddressWSNotAllowed
24	Error - whitespace in address '%.30s' is not allowed.
$ #CouldntDeleteData
25	Could not delete alias data file %s!
$ #CouldntDeleteHash
26	Could not delete alias hash file %s!
