
#define AttachSet	0x7
#define AttachMainMessage	0x1
#define AttachTooManyAttachments	0x2
#define AttachScreenTitle	0x3
#define AttachScreenPage	0x4
#define AttachCurrTitle	0x5
#define AttachCurrFileName	0x6
#define AttachCurrContType	0x7
#define AttachCurrContEncoding	0x8
#define AttachCurrContDescription	0x9
#define AttachCurrContDisposition	0xa
#define AttachMainInstrNorm	0xb
#define AttachMainInstrDummy1	0xc
#define AttachMainInstrDummy2	0xd
#define AttachMainPrompt	0xe
#define AttachAttachment	0xf
#define AttachNoAttachmentThere	0x10
#define AttachSelectionNotChanged	0x11
#define AttachNoRoom	0x12
#define AttachNewAttachment	0x13
#define AttachAttachmentNotAdded	0x14
#define AttachAttachmentHasBeenAdded	0x15
#define AttachYouCantDelete	0x16
#define AttachReallyDelete	0x17
#define AttachAttachmentNotDeleted	0x18
#define AttachAttachmentHasBeenDeleted	0x19
#define AttachYouCantChange	0x1a
#define AttachNotChanged	0x1b
#define AttachAttachmentFileChanged	0x1c
#define AttachAttachmentContTypeChanged	0x1d
#define AttachAttachmentContEncodingChanged	0x1e
#define AttachAttachmentContDescriptionChanged	0x1f
#define AttachAttachmentContDispositionChanged	0x20
#define AttachYouCantTag	0x21
#define AttachAttachmentHasBeenTagged	0x22
#define AttachTagHasBeenRemoved	0x23
#define AttachTagsAlreadyClear	0x24
#define AttachReallyClearTags	0x25
#define AttachTagsNotChanged	0x26
#define AttachTagsHaveBeenCleared	0x27
#define AttachMustTagAttachments	0x28
#define AttachMoveTaggedAttachments	0x29
#define AttachMoveTaggedAttachment	0x2a
#define AttachAttachmentsNotMoved	0x2b
#define AttachAttachmentNotMoved	0x2c
#define AttachAttachmentsHaveBeenMoved	0x2d
#define AttachAttachmentHaveBeenMoved	0x2e
#define AttachChgFileInstrNorm	0x2f
#define AttachChgFileInstrDummy1	0x30
#define AttachChgFileInstrDummy2	0x31
#define AttachChgTypeInstrNorm	0x32
#define AttachChgTypeInstrDummy1	0x33
#define AttachChgTypeInstrDummy2	0x34
#define AttachChgTypeEnterType	0x35
#define AttachChgEncodingInstrNorm	0x36
#define AttachChgEncodingInstrDummy1	0x37
#define AttachChgEncodingInstrDummy2	0x38
#define AttachChgDescripInstrNorm	0x39
#define AttachChgDescripInstrDummy1	0x3a
#define AttachChgDescripInstrDummy2	0x3b
#define AttachChgDispositionInstrNorm	0x3c
#define AttachChgDispositionInstrDummy1	0x3d
#define AttachChgDispositionInstrDummy2	0x3e
