
#define FbrowserSet	0x6
#define FbrowserCannotBrowse	0x1
#define FbrowserHeaderPageOf	0x2
#define FbrowserHeaderDirectory	0x3
#define FbrowserHeaderPattern	0x4
#define FbrowserInstNorm	0x5
#define FbrowserInstrDummy1	0x6
#define FbrowserInstrDummy2	0x7
#define FbrowserMainPrompt	0x8
#define FbrowserNoDirectorySaved	0x9
#define FbrowserAlreadyIn	0xa
#define FbrowserLimitInstrNorm	0xb
#define FbrowserLimitInstrDummy1	0xc
#define FbrowserLimitInstrDummy2	0xd
#define FbrowserLimitPrompt	0xe
#define FbrowserNotChanged	0xf
#define FbrowserDirInstrNorm	0x10
#define FbrowserDirInstrDummy1	0x11
#define FbrowserDirInstrDummy2	0x12
#define FbrowserDirPrompt	0x13
#define FbrowserEnterInstrNorm	0x14
#define FbrowserEnterInstrDummy1	0x15
#define FbrowserEnterInstrDummy2	0x16
#define FbrowserEnterPrompt	0x17
#define FbrowserOptionsTitle	0x18
#define FbrowserOptionsTitleDotf	0x19
#define FbrowserOptionsOn	0x1a
#define FbrowserOptionsOff	0x1b
#define FbrowserOptionsSpaceToToggle	0x1c
#define FbrowserOptionsTitleSort	0x1d
#define FbrowserOptionsNextOrReverse	0x1e
#define FbrowserOptionsInstructDotf	0x1f
#define FbrowserOptionsInstructSort	0x20
#define FbrowserOptionsInstructMain	0x21
#define FbrowserOptionsPromptCommand	0x22
#define FbrowserOptionsBogusSel	0x23
#define FbrowserOptionsNotChanged	0x24
#define FbrowserOptionsChangesUndone	0x25
#define FbrowserEnthdrPermission	0x26
#define FbrowserEnthdrType	0x27
#define FbrowserEnthdrOwner	0x28
#define FbrowserEnthdrSize	0x29
#define FbrowserEntryDateFmt	0x2a
#define FbrowserEnthdrDate	0x2b
#define FbrowserEnthdrFilename	0x2c
#define FbrowserEntryFile	0x2d
#define FbrowserEntryDir	0x2e
#define FbrowserEntryCurrentDir	0x2f
#define FbrowserEntryParentDir	0x30
#define FbrowserEnterdirCannotRead	0x31
#define FbrowserEnterdirScanning	0x32
#define FbrowserEnterdirRescan	0x33
#define FbrowserEnterdirScan	0x34
#define FbrowserEnterdirSelectedAll	0x35
#define FbrowserEnterdirSelectedEntries	0x36
#define FbrowserSorttypeReverse	0x37
#define FbrowserSorttypeName	0x38
#define FbrowserSorttypeSize	0x39
#define FbrowserSorttypeDate	0x3a
#define FbrowserSorttypeUnknown	0x3b
#define FbrowserMboxCannotGetStatus	0x3c
#define FbrowserMboxNotRegularFile	0x3d
#define FbrowserMboxNoPermissionRead	0x3e
#define FbrowserMboxNoPermissionWrite	0x3f
#define FbrowserNotValidMailbox	0x40
#define FbrowserSafeCopyOverflow	0x41
