
#define AnswerSet	0xb
#define AnswerQuitWord	0x1
#define AnswerExitWord	0x2
#define AnswerDoneWord	0x3
#define AnswerByeWord	0x4
#define AnswerMessageTo	0x5
#define AnswerSorryNotFound	0x6
#define AnswerCouldNotOpenWrite	0x7
#define AnswerCaller	0x8
#define AnswerOf	0x9
#define AnswerPhone	0xa
#define AnswerTelephoned	0xb
#define AnswerCalledToSeeYou	0xc
#define AnswerWantsToSeeYou	0xd
#define AnswerReturnedYourCall	0xe
#define AnswerPleaseCall	0xf
#define AnswerWillCallAgain	0x10
#define AnswerUrgent	0x11
#define AnswerEnterMessage	0x12
#define AnswerElmCommand	0x13
#define AnswerCannotHaveMoreNames	0x14
#define AnswerNotFoundForGroup	0x15
#define AnswerRecursionTooDeep	0x16
