
#define FromSet	0x11
#define FromForMoreInfo	0x1
#define FromNoQuietVerbose	0x2
#define FromCantOpenDevNull	0x3
#define FromNoPasswdEntry	0x4
#define FromCouldntExpandFilename	0x5
#define FromNotRegularFile	0x6
#define FromNoMail	0x7
#define FromCouldntOpenFolder	0x8
#define FromCouldntOpenFolderPlural	0x9
#define FromStringNoMail	0xa
#define FromNoMesgInFolder	0xb
#define FromNoExplainMail	0xc
#define FromNoExplainMessages	0xd
#define FromStringStringMail	0xe
#define FromThereAreMesg	0xf
#define FromFollowingMesg	0x10
#define FromFolderContainsFollowing	0x11
#define FromFolderContains	0x12
#define FromMessage	0x13
#define FromMessagePlural	0x14
#define FromNoMessages	0x15
#define FromAnon	0x16
#define FromYouHave	0x17
#define FromHas	0x18
#define FromUsage	0x19
#define FromHelpTitle	0x1a
#define FromHelpText	0x1b
#define FromNew	0x1c
#define FromUnread	0x1d
#define FromRead	0x1e
#define FromNewAndUnread	0x1f
#define FromNewOrUnread	0x20
#define FromNewAndRead	0x21
#define FromNewOrRead	0x22
#define FromReadAndUnread	0x23
#define FromReadOrUnread	0x24
#define FromUnknown	0x25
